package dev.shadowsoffire.apotheosis.adventure.socket.gem;

import net.minecraft.ResourceLocationException;
import net.minecraft.resources.ResourceLocation;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import java.util.Iterator;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.NbtUtils;
import net.minecraft.nbt.Tag;
import java.util.ArrayList;
import net.minecraft.nbt.CompoundTag;
import java.util.Collections;
import java.util.UUID;
import org.jetbrains.annotations.Nullable;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import net.minecraft.world.level.ItemLike;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import java.util.function.Function;
import java.util.Comparator;
import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.damagesource.DamageTypes;
import net.minecraft.world.damagesource.DamageSource;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.network.chat.Style;
import java.util.Objects;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import net.minecraft.ChatFormatting;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.network.chat.Component;
import java.util.List;
import net.minecraft.world.level.Level;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.placebo.tabs.ITabFiller;
import net.minecraft.world.item.Item;

public class GemItem extends Item implements ITabFiller
{
    public static final String HAS_REFRESHED = "has_refreshed";
    public static final String UUID_ARRAY = "uuids";
    public static final String GEM = "gem";
    
    public GemItem(final Item.Properties pProperties) {
        super(pProperties);
    }
    
    public void m_7373_(final ItemStack pStack, final Level pLevel, final List<Component> tooltip, final TooltipFlag pIsAdvanced) {
        final GemInstance inst = GemInstance.unsocketed(pStack);
        if (!inst.isValidUnsocketed()) {
            tooltip.add((Component)Component.m_237113_("Errored gem with no bonus!").m_130940_(ChatFormatting.GRAY));
            return;
        }
        final Gem gem = (Gem)inst.gem().get();
        final LootRarity rarity = (LootRarity)inst.rarity().get();
        Objects.requireNonNull(tooltip);
        gem.addInformation(pStack, rarity, tooltip::add);
    }
    
    public Component m_7626_(final ItemStack pStack) {
        final GemInstance inst = GemInstance.unsocketed(pStack);
        if (!inst.isValidUnsocketed()) {
            return super.m_7626_(pStack);
        }
        MutableComponent comp = Component.m_237115_(this.m_5671_(pStack));
        comp = Component.m_237110_("item.apotheosis.gem." + inst.rarity().getId(), new Object[] { comp });
        return (Component)comp.m_130948_(Style.f_131099_.m_131148_(((LootRarity)inst.rarity().get()).getColor()));
    }
    
    public String m_5671_(final ItemStack pStack) {
        final DynamicHolder<Gem> gem = getGem(pStack);
        if (!gem.isBound()) {
            return super.m_5524_();
        }
        return super.m_5671_(pStack) + "." + gem.getId();
    }
    
    public boolean m_5812_(final ItemStack pStack) {
        final GemInstance inst = GemInstance.unsocketed(pStack);
        if (!inst.isValidUnsocketed()) {
            return super.m_5812_(pStack);
        }
        return inst.isMaxRarity();
    }
    
    public boolean m_41386_(final DamageSource src) {
        return super.m_41386_(src) && !src.m_276093_(DamageTypes.f_268526_);
    }
    
    public void fillItemCategory(final CreativeModeTab group, final CreativeModeTab.Output out) {
        GemRegistry.INSTANCE.getValues().stream().sorted(Comparator.comparing((Function<? super T, ? extends Comparable>)Gem::getId)).forEach(gem -> RarityRegistry.INSTANCE.getOrderedRarities().stream().map(DynamicHolder::get).forEach(rarity -> {
            if (gem.clamp(rarity) == rarity) {
                final ItemStack stack = new ItemStack((ItemLike)this);
                setGem(stack, gem);
                AffixHelper.setRarity(stack, rarity);
                out.m_246342_(stack);
            }
        }));
    }
    
    @Nullable
    public String getCreatorModId(final ItemStack stack) {
        final GemInstance inst = GemInstance.unsocketed(stack);
        if (inst.isValidUnsocketed()) {
            return inst.gem().getId().m_135827_();
        }
        return super.getCreatorModId(stack);
    }
    
    public static List<UUID> getUUIDs(final ItemStack gemStack) {
        final DynamicHolder<Gem> gem = getGem(gemStack);
        if (!gem.isBound()) {
            return Collections.emptyList();
        }
        return getOrCreateUUIDs(gemStack.m_41784_(), ((Gem)gem.get()).getNumberOfUUIDs());
    }
    
    public static List<UUID> getOrCreateUUIDs(final CompoundTag tag, final int numUUIDs) {
        if (numUUIDs == 0) {
            return Collections.emptyList();
        }
        if (!tag.m_128441_("uuids")) {
            return generateAndSave(new ArrayList<UUID>(numUUIDs), numUUIDs, tag);
        }
        final ListTag list = tag.m_128437_("uuids", 11);
        final List<UUID> ret = new ArrayList<UUID>(list.size());
        for (final Tag t : list) {
            ret.add(NbtUtils.m_129233_(t));
        }
        if (ret.size() < numUUIDs) {
            return generateAndSave(ret, numUUIDs, tag);
        }
        return ret;
    }
    
    private static List<UUID> generateAndSave(final List<UUID> base, final int amount, final CompoundTag tag) {
        for (int needed = amount - base.size(), i = 0; i < needed; ++i) {
            base.add(UUID.randomUUID());
        }
        final ListTag list = new ListTag();
        for (final UUID id : base) {
            list.add((Object)NbtUtils.m_129226_(id));
        }
        tag.m_128365_("uuids", (Tag)list);
        return base;
    }
    
    public static void setGem(final ItemStack gemStack, final Gem gem) {
        gemStack.m_41784_().m_128359_("gem", gem.getId().toString());
    }
    
    public static DynamicHolder<Gem> getGem(final ItemStack gem) {
        if (gem.m_41720_() != Adventure.Items.GEM.get() || !gem.m_41782_()) {
            return (DynamicHolder<Gem>)GemRegistry.INSTANCE.emptyHolder();
        }
        try {
            final CompoundTag tag = gem.m_41783_();
            return (DynamicHolder<Gem>)GemRegistry.INSTANCE.holder(new ResourceLocation(tag.m_128461_("gem")));
        }
        catch (final ResourceLocationException ex) {
            return (DynamicHolder<Gem>)GemRegistry.INSTANCE.emptyHolder();
        }
    }
}
